//     Author: Maurizio Loreti, aka MLO or (HAM) I3NOO
//     Work:   University of Padova - Department of Physics
//             Via F. Marzolo, 8 - 35131 PADOVA - Italy
//     Phone:  ++39(49) 827-7216     FAX: ++39(49) 827-7102
//     EMail:  loreti@padova.infn.it
//     WWW:    http://wwwcdf.pd.infn.it/~loreti/mlo.html
//
// -------------------------------------------------------------
//
//     File: file.cxx - Called by "texref" to process each
//           file.
//   
//     $Id: file.cxx,v 1.5 2005/01/21 08:21:57 loreti Exp $

#include "texref.hh"
#include "file.hh"

using std::cout;
using std::endl;
using std::set;
using std::string;

// Local procedures

static void insert(const string &, const Position &,
                   set<Keyword>::const_iterator &);

void processFile(
  std::ifstream * fp,
  const string  & fName
) {

  unsigned long lineNo   = 0;
  const int     LINE_LEN = 128;

  char line[LINE_LEN];
 
// Reads every input line, skipping TeX comments.
// The test is not completely correct; a keyword may be commented out
// also if the comment does not start in column 1.  But I cannot perform
// a complete analysis of the TeX statements here, and this test filters
// out most of the commented keywords.

  while (fp->getline(line, LINE_LEN) != 0) {
    ++lineNo;

    if (line[0] == '%') continue;

    string sLine(line);
    set<Keyword>::const_iterator it, finis = pKeys->end();

    for (it = pKeys->begin();   it != finis;   ++it) {
      size_t offset, sub;

// Loops over all the known keywords, looking for their occurrence in
// the current line; a keyword may appear more than once in every line.

      for (offset = 0;
           (sub = sLine.find(*(it->getName()), offset)) != string::npos;
           offset = sub
      ) {
        Position pos(fName, lineNo);
        size_t start = sub + it->getLen();
        size_t end   = sLine.find('}', start);
        
        if (end == string::npos) {
          cout << progName << ": unterminated keyword \""
               << *(it->getName()) << "\" in " << pos << endl;
          break;
        } else {
                string  field = sLine.substr(start, end-start);
          const string *pSep  = it->getSep();

// If a keyword may reference multiple labels (then pSep is not NULL),
// we split its argument accordingly.  "insert" inserts the occurrence
// of the referenced labels in the summary.
          
          if (pSep == 0) {
            insert(field, pos, it);
          } else {
            size_t subStart = 0;
                        
            for (;;) {
              size_t subEnd = field.find_first_of(*pSep, subStart);

              if (subEnd == string::npos) break;

              string subField = field.substr(subStart, subEnd-subStart);
              insert(subField, pos, it);
              subStart = subEnd + 1;

            } // for loop (subfields)

            insert(field.substr(subStart), pos, it);
           } // if (multiple arguments)

          sub = end + 1;
        } // if (end of keyword)
      }   // for loop (offset)
    }     // for loop (keywords)
  }       // while loop (getline)

  cout << progName << ": " << lineNo << " lines read from "
       << fName << endl;
}

static void insert(
  const string                 & name,
  const Position               & pos,
  set<Keyword>::const_iterator & it
) {
  std::pair< set<Label>::iterator, bool > pib = pLabs->insert( Label(name) );
  (pib.first)->addData(pos, it->getType(), it->getFamily());
}

