//     Author: Maurizio Loreti, aka MLO or (HAM) I3NOO
//     Work:   University of Padova - Department of Physics
//             Via F. Marzolo, 8 - 35131 PADOVA - Italy
//     Phone:  ++39(49) 827-7216     FAX: ++39(49) 827-7102
//     EMail:  loreti@padova.infn.it
//     WWW:    http://wwwcdf.pd.infn.it/~loreti/mlo.html
//
// -------------------------------------------------------------
//
//     File: texref.cxx - Methods for the classes defined in
//           texref.h.
//   
//     $Id: texref.cxx,v 1.7 2001/08/16 18:12:29 loreti Exp $

#include "texref.hh"

using std::list;
using std::ostream;
using std::set;
using std::string;

// Methods for class "Position"

ostream & operator << (
        ostream  & o,
  const Position & pos
) {
  o << "file " << pos._file << ", line " << pos._line;
  return o;
}

// Methods for class "Keyword"

Keyword::Keyword(
  const string    & name,
  const string    * pSeps,
  const KeyType     t,
  const KeyFamily   f
) : _name(name), _type(t), _fam(f) {
  if (pSeps == 0   ||   pSeps->empty()) {
    _psep = 0;
  } else {
    _psep = new string(*pSeps);
  }
}

Keyword::Keyword(
  const char      * pName,
  const char      * pSeps,
  const KeyType     t,
  const KeyFamily   f
) : _name(pName), _type(t), _fam(f) {
  if (pSeps == 0   ||   *pSeps == '\0') {
    _psep = 0;
  } else {
    _psep = new string(pSeps);
  }
}

Keyword::Keyword(
  const Keyword & rhs
) : _name(rhs._name), _type(rhs._type), _fam(rhs._fam) {
  _psep = rhs._psep ? new string( *(rhs._psep) ) : 0;
}

const Keyword & Keyword::operator = (
  const Keyword & rhs
) {
  if (this != &rhs) {
    delete _psep;

    _name = rhs._name;
    _type = rhs._type;
    _fam  = rhs._fam;
    _psep = rhs._psep ? new string( *(rhs._psep) ) : 0;
  }
  return *this;
}

// Methods for class "Label"

Label::Label(
  const string & name
) : _name(name) {
  _pfam = new set<KeyFamily>;
  _pdef = new list<Position>;
  _pref = new list<Position>;
}

Label::Label(
  const char * pName
) : _name(pName) {
  _pfam = new set<KeyFamily>;
  _pdef = new list<Position>;
  _pref = new list<Position>;
}

Label::Label(
  const Label & rhs
) : _name(rhs._name) {
  _pfam = rhs._pfam ? new set<KeyFamily>( *(rhs._pfam) ) : 0;
  _pdef = rhs._pdef ? new list<Position>( *(rhs._pdef) ) : 0;
  _pref = rhs._pref ? new list<Position>( *(rhs._pref) ) : 0;
}

const Label & Label::operator = (
  const Label & rhs
) {
  if (this != &rhs) {
    delete _pfam;
    delete _pdef;
    delete _pref;

    _name = rhs._name;
    _pfam = rhs._pfam ? new set<KeyFamily>( *(rhs._pfam) ) : 0;
    _pdef = rhs._pdef ? new list<Position>( *(rhs._pdef) ) : 0;
    _pref = rhs._pref ? new list<Position>( *(rhs._pref) ) : 0;
  }
  return *this;
}
void Label::addData(
  const Position  pos,
        KeyType   k,
        KeyFamily f
) const {
  _pfam->insert(f);

  switch (k) {
    case DEFINITION:
      _pdef->push_back(pos);
      break;
    case REFERENCE:
      _pref->push_back(pos);
      break;
  }
}
