#include <stdlib.h>
#include <string.h>
#include "texref.h"
#include "file.h"
#include "tree.h"

void dealWithFile(void)
{
  char buffer[LINE_MAX];        /* Input buffer */
  int isDef;                    /* TRUE: definitions; FALSE: references */
  const LabelClass *pLC;
  const Keyword    *pK;

/**
 | Reads every input line, skipping TeX comments.
 | The test is not completely correct; a keyword may be commented out
 | also if the comment does not start in column 1.  But I cannot perform
 | a complete analysis of the TeX statements here, and this test filters
 | out most of the commented keywords.
**/

  while (fgets(buffer, LINE_MAX, pIn) != 0) {
    lineNum++;
    if (buffer[0] == '%') continue;

/**
 | Loops over all the label families, looking for occurrences of their
 | keywords in the current line.  A keyword may appear more than once:
 | for every occurrence, pStart and pEnd will point to the first
 | character of the label name, and just after the label itself; *pEnd
 | is temporarily changed to ASCII NUL, then restored.
**/

    pLC   = ClassTree;
    isDef = TRUE;
    pK    = pLC->def;

    while (pLC != 0) {
      char *pc;

      for (pc = buffer;   pc != 0;   ) {
        char *pStart, *pEnd;

        if ((pStart = strstr(pc, pK->name)) != 0) {
          pStart += pK->length;

          if ((pEnd = strchr(pStart, '}')) == 0) {
            fprintf(stderr,
                    "%s: unterminated keyword \"%s\" in file %s, line %lu\n",
                    progName, pK->name, fName, lineNum);
            goto nextKey;
          } else {
            size_t length = pEnd - pStart;

            *pEnd = '\0';

/**
 | If a keyword may reference multiple labels (pK->list is not NULL),
 | we create a temporary copy of the keyword argument --- that will
 | be split with strtok().  doTheStuff inserts the occurrence of the
 | keyword in the linked list of the found labels.
**/

            if (pK->list == 0) {
              if (doTheStuff(isDef, pStart, length) == FAILURE) {
                cleanup();
                exit(EXIT_FAILURE);
              }
            } else {
              char    temp[LINE_MAX];
              char   *pSub;
              size_t  lSub;
              
              strcpy(temp, pStart);

              pSub = strtok(temp, pK->list);
              do {
                lSub = strlen(pSub);
                if (doTheStuff(isDef, pSub, lSub) == FAILURE) {
                  cleanup();
                  exit(EXIT_FAILURE);
                }
                pSub = strtok(0, pK->list);
              } while (pSub != 0);
            } /* if (multiple references allowed) */
            
            *pEnd = '}';
          } /* if (a closing '}' has been found) */
          
          pc = pEnd + 1;

        } else {
          pc = 0;
        } /* if (keyword found) */
      } /* pLC loop over label classes */
      
/**
 | We checked for this keyword; now, get the next one.
**/

    nextKey:
      if (isDef) {
        isDef = FALSE;
        pK = pLC->ref;
      } else {
        pK = pK->next;

        if (pK == 0) {
          pLC = pLC->next;

          if (pLC != 0) {
            pK    = pLC->def;
            isDef = TRUE;
          }
        }
      }
    }
  }

/**
 | Checks for a real end-of-file, or a read error; then closes
 | the current file and returns.
**/

  if (ferror(pIn)) {
    fprintf(stderr, "%s: read error from file \"%s\"\n", progName, fName);
    perror(progName);
    cleanup();
    exit(EXIT_FAILURE);
  }
  
  fclose(pIn);
  pIn = 0;
}
