#ifndef TEXREF_H_
#define TEXREF_H_

#include <stdio.h>

/**
 | Definitions:
 | - maximum line length fot input files;
 | - two pairs of symbolic names internally used.
**/

#define LINE_MAX     256

#define FAILURE        0
#define SUCCESS        1

#define TRUE           1
#define FALSE          0

/**
 | Label reference/definition:
 | - a keyword used to reference or define a label in a given context;
 | - the length of that keyword;
 | - a string of separators for multiple references within the given
 |   keyword, or NULL if multiple references are not allowed;
 | - a pointer to the next keyword (in a linked list).
**/

typedef struct sKeyword {
  const size_t           length;
  const struct sKeyword *next;
  const char            *name;
  const char            *list;
} Keyword;

/**
 | Label class context:
 | - an unique keyword to define the label;
 | - one or more keywords to reference that label;
 | - a pointer to the next class context (in a linked list).
**/

typedef struct sLabelClass {
  const Keyword            *def;
  const Keyword            *ref;
  const struct sLabelClass *next;
} LabelClass;

/**
 | Where a given string has been found:
 | - a file name;
 | - a line number within that file;
 | - and a pointer to the next occurrence (in a linked list).
**/

typedef struct sReference {
  unsigned long      lineNo;
  struct sReference *next;
  char               fileName[1];
} Reference;

/**
 | For every label:
 | - its name;
 | - where has been defined;
 | - where has been referenced;
 | - a pointer to the next label (in a linked list).
**/

typedef struct sLabel {
  Reference     *defined;
  Reference     *referenced;
  struct sLabel *next;
  char           name[1];
} Label;

/**
 | Global variables:
 | - the label families to look for (ClassTree);
 | - a list of all the found labels (LabelTree);
 | - the name (fName) of the file currently being read, and a FILE
 |   pointer used for access (pIn);
 | - the number of the line currently in the input buffer (lineNum).
**/

extern const LabelClass    *ClassTree;
extern       Label         *LabelTree;
extern       char          *fName;
extern       FILE          *pIn;
extern const char          *progName;
extern       unsigned long  lineNum;

/**
 | Procedure prototypes
**/

void  cleanup(void);

#endif /* TEXREF_H_ */
